package de.ullisroboterseite.ursai2notification;

import java.net.URI;
import java.util.*;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.app.*;
import android.content.*;

import android.media.AudioAttributes;
import android.media.AudioAttributes.Builder;
import android.media.RingtoneManager;

import android.os.Handler;
import android.os.Build;

import android.net.Uri;

import android.util.Log;

@SimpleObject(external = true)
public class ChannelProperties extends AndroidNonvisibleComponent {
    static final String LOG_TAG = "NOTIFY";
    Context thisContext;
    Activity thisActivity;
    Component thisComponent; // Wird im Consructor von UrsAI2NotificationChannel belegt
    final Handler handler = new Handler();

    NotificationManager manager;

    // Properties' fields
    boolean lightEnabled = false;
    int lightColor = Component.COLOR_DEFAULT;
    String description = "";
    String groupId = "";
    int importance = 3;
    String name = "Urs Notification Channel";
    boolean showBadge = false;
    public String channelID = "UrsNotificationChannel";
    boolean vibrationEnabled = false;
    long[] vibrationPattern = {};
    boolean vibrationPatternValid = true;
    boolean soundEnabled = false;
    String soundPath = "";
    boolean soundPathValid = true;
    boolean bypassDnd = false;
    int lockscreenVisibility = -1000;

    boolean channelOK = false; // true, der Kanal wurde erfolgreich angelegt.

    public ChannelProperties(ComponentContainer container) {
        super(container.$form());
        thisContext = container.$context();
        thisActivity = (Activity) container.$context();
        manager = (NotificationManager) thisContext.getSystemService(Context.NOTIFICATION_SERVICE);
    }

    @SimpleProperty(description = "The SDK version of the software currently running on this hardware device.")
    public int VersionSDK() {
        return Build.VERSION.SDK_INT;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#NotificationChannel(java.lang.String,%20java.lang.CharSequence,%20int)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_TEXT, alwaysSend = true, defaultValue = "UrsNotificationChannel")
    @SimpleProperty(description = "Sets the id of the channel. Must be unique per package. The value may be truncated if it is too long.", userVisible = false)
    public void ChannelID(String channelID) {
        channelID = channelID.trim();
        if (!channelID.isEmpty())
            this.channelID = channelID.trim();
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setBypassDnd(boolean)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, alwaysSend = true, defaultValue = "False")
    @SimpleProperty(description = "Notifications posted to this channel can interrupt the user in NotificationManager.INTERRUPTION_FILTER_PRIORITY mode", userVisible = false)
    public void BypassDnd(boolean value) {
        bypassDnd = value;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setName(java.lang.CharSequence)
    // änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_TEXT, alwaysSend = true, defaultValue = "Urs Notification Channel")
    @SimpleProperty(description = "The the user visible name of this channel.")
    public void ChannelName(String value) {
        value = value.trim();
        if (!value.isEmpty()) {
            name = value;
            if (channelOK) // nur nachträglich
                createChannel();
        }
    }

    @SimpleProperty(description = "The the user visible name of this channel.")
    public String ChannelName() {
        return name;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setDescription(java.lang.String)
    // änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_TEXT, alwaysSend = true, defaultValue = "")
    @SimpleProperty(description = "The user visible description of this channel.")
    public void Description(String value) {
        this.description = value;
        if (channelOK) // nur nachträglich
            createChannel();
    }

    @SimpleProperty(description = "The user visible description of this channel.")
    public String Description() {
        return description;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setImportance(int)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_CHOICES //
            , alwaysSend = true//
            , defaultValue = "default" //
            , editorArgs = { "none", "min", "low", "default", "high" })
    @SimpleProperty(description = "The level of interruption of this notification channel.", userVisible = false)
    public void Importance(String value) {
        if (value.equals("none"))
            importance = NotificationManager.IMPORTANCE_NONE;
        else if (value.equals("min"))
            importance = NotificationManager.IMPORTANCE_MIN;
        else if (value.equals("low"))
            importance = NotificationManager.IMPORTANCE_LOW;
        else if (value.equals("default"))
            importance = NotificationManager.IMPORTANCE_DEFAULT;
        else if (value.equals("high"))
            importance = NotificationManager.IMPORTANCE_HIGH;
        else
            importance = NotificationManager.IMPORTANCE_UNSPECIFIED;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#shouldShowLights()
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, alwaysSend = true, defaultValue = "False")
    @SimpleProperty(description = "Sets whether notifications posted to this channel should display notification lights, "
            + "on devices that support that feature.", userVisible = false)
    public void LightEnabled(boolean value) {
        lightEnabled = value;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setLightColor(int)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COLOR, alwaysSend = true, defaultValue = Component.DEFAULT_VALUE_COLOR_DEFAULT)
    @SimpleProperty(description = "Sets the notification light color for notifications posted to this channel, "
            + "if lights are enabled on this channel and the device supports that feature.", userVisible = false)
    public void LightColor(int argb) {
        lightColor = argb;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setLockscreenVisibility(int)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_CHOICES //
            , alwaysSend = true//
            , defaultValue = "none" //
            , editorArgs = { "none", "secret", "private", "public" })
    @SimpleProperty(description = "Sets whether notifications posted to this channel appear on the lockscreen or not, "
            + "and if so, whether they appear in a redacted form.", userVisible = false)
    public void LockscreenVisibility(String value) {
        if (value.equals("secret"))
            lockscreenVisibility = Notification.VISIBILITY_SECRET;
        else if (value.equals("private"))
            lockscreenVisibility = Notification.VISIBILITY_PRIVATE;
        else if (value.equals("public"))
            lockscreenVisibility = Notification.VISIBILITY_PUBLIC;
        else
            lockscreenVisibility = -1000; // Dies ist der Wert, ohne dass die Eigenschaft gesetzt wird.
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setShowBadge(boolean)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, alwaysSend = true, defaultValue = "False")
    @SimpleProperty(description = "Sets whether notifications posted to this channel can appear as application icon badges in a Launcher.", userVisible = false)
    public void ShowBadge(boolean value) {
        showBadge = value;
    }

    /* --- Sound klappt nicht, weil der AssetFileProvider nicht richtig registiert werden kann.
    // https://developer.android.com/reference/android/app/NotificationChannel#setSound(android.net.Uri,%20android.media.AudioAttributes)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_ASSET, defaultValue = "")
    @SimpleProperty(description = "The sound that should be played for notifications posted to this channel.", userVisible = false)
    public void Sound(String value) {
        soundPath = value != null ? value : "";
        soundPath = soundPath.trim();
        if (soundPath.isEmpty())
            return;

        if (!Util.assetFileExists(form, soundPath))
            soundPathValid = false;
    }
    */

    // https://developer.android.com/reference/android/app/NotificationChannel#shouldShowLights()
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, alwaysSend = true, defaultValue = "False")
    @SimpleProperty(description = "Sets whether notifications posted to this channel should play the default sound.", userVisible = false)
    public void SoundEnabled(boolean value) {
        soundEnabled = value;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#enableVibration(boolean)
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, alwaysSend = true, defaultValue = "False")
    @SimpleProperty(description = "Sets whether notification posted to this channel should vibrate.", userVisible = false)
    public void VibrationEnabled(boolean value) {
        vibrationEnabled = value;
    }

    // https://developer.android.com/reference/android/app/NotificationChannel#setVibrationPattern(long[])
    // nicht änderbar
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_TEXTAREA, defaultValue = "")
    @SimpleProperty(description = "Sets the vibration pattern for notifications posted to this channel.", userVisible = false)
    public void VibrationPattern(String itemString) {
        if (itemString.trim().isEmpty())
            return;

        itemString.replace(";", ",");
        String[] items = itemString.split(",");

        long[] vp = new long[items.length];
        for (int i = 0; i < items.length; i++) {
            try {
                vp[i] = Integer.parseInt(items[i].trim());
            } catch (Exception e) {
                vibrationPatternValid = false;
                return;
            }
        }
        vibrationPattern = vp;
    }

    // ========================================================
    void createChannel() {
        final Component component = thisComponent;

        if (Build.VERSION.SDK_INT < 26) { // Vor Version 26 gab es keine Kanäle
            channelOK = true;
            Log.d(LOG_TAG, "Kein Channel angelegt");
            return;
        }

        NotificationChannel chn = manager.getNotificationChannel(channelID);
        if (chn != null)
            channelOK = true;

        if (!vibrationPatternValid && !channelOK) { // Den Kanal gibt es ggf. bereits.
            handler.post(new Runnable() {
                public void run() {
                    form.ErrorOccurred(component, "Start", 17002, "Invalid Vibration Pattern");
                } // run
            }); // post
            return;
        }

        if (!soundPathValid && !channelOK) { // Den Kanal gibt es ggf. bereits.
            handler.post(new Runnable() {

                public void run() {
                    form.ErrorOccurred(component, "Start", 17003, "Invalid sound file name");
                } // run
            }); // post
            form.ErrorOccurred(component, "Start", 999, "Valid sound file name");
            return;
        }

        NotificationChannel notificationChannel = new NotificationChannel(channelID, name, importance);
        notificationChannel.enableLights(lightEnabled);
        notificationChannel.setLightColor(lightColor);
        notificationChannel.setDescription(description);
        notificationChannel.setShowBadge(showBadge);
        notificationChannel.setLockscreenVisibility(lockscreenVisibility);
        notificationChannel.setBypassDnd(bypassDnd);

        notificationChannel.enableVibration(vibrationEnabled);
        if (vibrationPattern.length > 0)
            notificationChannel.setVibrationPattern(vibrationPattern);

        AudioAttributes audioAttributes = new AudioAttributes.Builder() //
                .setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION) //
                .setUsage(AudioAttributes.USAGE_NOTIFICATION) //
                .build();

        /*
        if (!soundPath.isEmpty()) { // Das kann doch so nicht stimmen, oder? Die Assets gehören zur App!
            Uri uri = Uri.parse("content://de.ullisroboterseite.ursai2notification/" + soundPath);
            if (form instanceof ReplForm) {
                if (((ReplForm) form).isAssetsLoaded()) {
                    uri = android.net.Uri.parse(URI.create(form.getAssetPath(soundPath)).toString());
                }
            }
            notificationChannel.setSound(uri, audioAttributes);
        } else
        */
        if (soundEnabled)
            notificationChannel.setSound(RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION),
                    audioAttributes);

        manager.createNotificationChannel(notificationChannel);
        Log.d(LOG_TAG, "Channel created");
    }

    @SimpleFunction(description = "Sorts a simple list in ascending or descending.")
    public YailList SortSimpleList(YailList List, boolean Ascending) {
        String[] strArray = List.toStringArray();
        if (Ascending) {
            Arrays.sort(strArray);
        } else {
            Arrays.sort(strArray, Collections.reverseOrder());
        }
        return YailList.makeList(strArray);
    }

}